@@ -1,4 +1,4 @@
-#' Detects unreliable outliers in univariate time series data based on 
+#' Detects unreliable outliers in univariate time series data based on
 #' model-based clustering
 #'
 #' @description This function applies finite mixture modelling to compute
@@ -6,7 +6,7 @@
 #' in an univariate time series.
 #' By utilizing the \link[mclust]{Mclust} package the data is
 #' assigned in G clusters whereof one is modelled as an outlier cluster.
-#' The clustering process is based on features, which are modelled to 
+#' The clustering process is based on features, which are modelled to
 #' differentiate normal from outlying observation.Beside computing
 #' the probability of each observation being outlying data also
 #' the specific cause in terms of the responsible feature/ feature combination
@@ -37,7 +37,7 @@
 #' share of outliers in data increases. A priori it is assumed that
 #' out.par * ceiling(sqrt(nrow(data.original))) number of observations are outlying observations.
 #' @param detection.parameter denotes a parameter to regulate the
-#' detection sensitivity. By default set to 1. It is assumed that the outlier cluster 
+#' detection sensitivity. By default set to 1. It is assumed that the outlier cluster
 #' follows a (multivariate) Gaussian distribution parameterized by sample mean and a blown up
 #' sample covariance matrix of the feature space. The covariance matrix is blown up
 #' by detection.parameter * (2 * log(length(data)))^2.
@@ -60,7 +60,7 @@
 #' For cluster estimation the \link[mclust]{Mclust} function is applied.
 #' Models are estimated by the EM algorithm initialized by hierarchical
 #' model-based agglomerative clustering. The optimal model is selected
-#' according to BIC.
+#' according to BIC. \insertNoCite{*}{tsrobprep}
 #' The following features based on the introduced data are used in the clustering process:
 #' \describe{
 #'   \item{org.series}{denotes the scaled and potantially decomposed original time series.}
@@ -68,8 +68,8 @@
 #'   \item{gradient}{denotes the summation of the two sided gradient of the org.series.}
 #'   \item{abs.gradient}{denotes the summation of the absolute two sided gradient of
 #'    org.series.}
-#'   \item{rel.gradient}{denotes the summation of the two sided absolute gradient of the 
-#'   org.series with sign based on left sided gradient in relation to the 
+#'   \item{rel.gradient}{denotes the summation of the two sided absolute gradient of the
+#'   org.series with sign based on left sided gradient in relation to the
 #'   rolling mean absolut deviation based on most relevant seasonality S.}
 #'   \item{abs.seas.grad}{denotes the summation of the absolute two sided seasonal gradient of
 #'   org.series based on seasonalties S.}
@@ -85,10 +85,10 @@
 #' being outlying data.}
 #' \item{Repetitions}{provides a list for each repetition containing the estimated model,
 #' the outlier cluster, the probabilities for each observation belonging to the estimated
-#' clusters, the outlier position, the influence of each feature/ feature combination 
-#' on the identified outyling data, and the corresponding probabilities 
-#' after shift to the feature mean of each considered outlier, as well as the applied 
-#' subset of the extended feature matrix for estimation (including artificially introduced 
+#' clusters, the outlier position, the influence of each feature/ feature combination
+#' on the identified outyling data, and the corresponding probabilities
+#' after shift to the feature mean of each considered outlier, as well as the applied
+#' subset of the extended feature matrix for estimation (including artificially introduced
 #' outliers).
 #' }
 #' \item{features}{a matrix containg the feature matrix. Each column is a feature.}
@@ -97,6 +97,9 @@
 #' \item{feature.inf.tab}{a matrix containing all possible feature combinations.}
 #' \item{PC}{an object of class "princomp" containing the principal component analysis
 #' of the feature matrix.}
+#' @references
+#' \insertAllCited{}
+#' @importFrom Rdpack reprompt
 #' @examples
 #' \dontrun{
 #' set.seed(1)
@@ -107,28 +110,28 @@
 #'                               consider.as.missing = 0, min.val = 0)
 #' # Impute missing values
 #' data.imputed <- impute_modelled_data(modelmd)
-#' 
+#'
 #' #Detect outliers
 #' system.time(
 #'   o.ident <- detect_outliers(data = data.imputed, S = c(48, 336))
 #' )
-#' 
+#'
 #' # Plot of identified outliers in time series
 #' outlier.vector <- rep(F,length(data.imputed))
 #' outlier.vector[o.ident$outlier.pos] <- T
 #' plot(data.imputed, type = "o", col=1 + 1 * outlier.vector,
 #'      pch = 1 + 18 * outlier.vector)
-#' 
+#'
 #' # table of identified raw outliers and corresponding probs being outlying data
 #' df <- data.frame(o.ident$outlier.pos.raw,unlist(o.ident$outlier.probs)[o.ident$outlier.pos.raw])
 #' colnames(df) <- c("Outlier position", "Probability of being outlying data")
 #' df
-#' 
+#'
 #' # Plot of feature matrix
 #' plot.ts(o.ident$features, type = "o",
 #'         col = 1 + outlier.vector,
 #'         pch = 1 + 1 * outlier.vector)
-#' 
+#'
 #' # table of outliers and corresponding features/ feature combinations,
 #' # which caused assignment to outlier cluster
 #' # Detect outliers with feat.int = T
@@ -138,7 +141,7 @@
 #' )
 #' feature.imp <- unlist(lapply(o.ident$inf.feature.combinations,
 #'                              function(x) paste(o.ident$feature.inf.tab[x], collapse = " | ")))
-#' 
+#'
 #' df <- data.frame(o.ident$outlier.pos.raw,o.ident$outlier.probs[o.ident$outlier.pos.raw],
 #'                  feature.imp[as.numeric(names(feature.imp)) %in% o.ident$outlier.pos.raw])
 #' colnames(df) <- c("Outlier position", "Probability being outlying data", "Responsible features")
@@ -171,22 +174,26 @@ detect_outliers <- function(data,
 
   # validate the variables' correctness - basic validation
   if(sum(is.na(data.original)) != 0) stop("Please impute missing data before usage.")
-  if(is.null(share)){
-    subset.length <- pmin(2*round(length(data.original)^(sqrt(2)/2)),
-                                  length(data.original))
-    # Set share to 1 if number of observations are smaller than 1000
-    if(subset.length < 1000){
-      share <- 1
-    } else {
-      share <- subset.length/length(data.original)
-    }
-  }
-  
+
+  if(is.null(share)) share <-pmin(2*round(length(data.original)^(sqrt(2)/2)),
+                                 length(data.original))/length(data.original)
+  # TODO
+  #if(is.null(share))
+  #  subset.length <- pmin(2*round(length(data.original)^(sqrt(2)/2)),
+  #                                length(data.original))
+  #  # Set share to 1 if number of observations are smaller than 1000
+  #  if(subset.length < 1000){
+  #    share <- 1
+  #  } else {
+  #    share <- subset.length/length(data.original)
+  #  }
+  #}
+
   if(is.null(G)){ # Create grid for search of clusters
     phi<- (sqrt(5)+1)/2
     G<- round((phi^(2+1:log(max.cluster-1, phi)))/sqrt(5) )
   }
-  
+
   ## TODO Disable seasonal features if data is too short
   if(sum(nrow(data.original) <= S) < length(S))  S <- S[which(S<nrow(data.original))]
 
@@ -215,8 +222,8 @@ detect_outliers <- function(data,
   ## Absolute two sided gradient
   abs.gradient <- abs(dyl) + abs(dyr) - abs(dyl + dyr) # Absolute gradient
   # Summation of dyl and dyr in accordance to sign(dyl)
-  pos.neg.gradient <- dyl + sign(dyl) * abs(dyr) 
-  # Computation of seasonal mad matrix based on strongest seasonality S 
+  pos.neg.gradient <- dyl + sign(dyl) * abs(dyr)
+  # Computation of seasonal mad matrix based on strongest seasonality S
   seas.mat <- matrix(, S[1], ceiling(length(pos.neg.gradient)/S[1]))
   seas.mat[1:length(pos.neg.gradient)]<- pos.neg.gradient
   seas.vol <- rep_len(apply(seas.mat, 1, stats::mad, na.rm=TRUE),
@@ -226,7 +233,7 @@ detect_outliers <- function(data,
   seas.vol.ma <- sapply(1:length(seas.vol), function(x)
     stats::median(seas.vol[pmax(x-H,1):pmin(x+H,length(seas.vol))]))
   # Add noise to aviod zero pos.neg.gradient
-  pos.neg.gradient_sc <- scale(pos.neg.gradient + 10^-100) 
+  pos.neg.gradient_sc <- scale(pos.neg.gradient + 10^-100)
   # Relative gradient
   # Add noise to aviod zero pos.neg.gradient
   rel.gradient <- as.numeric(pos.neg.gradient_sc / (seas.vol.ma + 10^-100))
@@ -241,9 +248,12 @@ detect_outliers <- function(data,
     dylS <- c(rep(0, S[i]), diff(org.series, S[i]))
     # Seasonal differences (y_{i+S} - y_{y-i})
     dyrS <- c(diff(org.series, S[i]), rep(0, S[i]))
-    #Absolute seasonal gradient
-    seas_feat_mat <- cbind(seas_feat_mat,
-                           abs(dylS) + abs(dyrS) - abs(dylS + dyrS))
+    ## Absolute two-sided seasonal gradient
+    asg <- abs(dylS) + abs(dyrS) - abs(dylS + dyrS)
+    # Add absolut one-sided seasonal gradient to start and end of series
+    asg_e <- c(utils::head(abs(dyrS), S[i]), asg[(S[i]+1):(length(asg)-S[i])], utils::tail(abs(dylS),S[i]))
+    # bind seasonal components
+    seas_feat_mat <- cbind(seas_feat_mat, asg_e)
     # Colnames
     names <- c(names, paste0("seasonality.", S[i]), paste0("abs.seas.grad.S", S[i]))
   }
@@ -251,7 +261,7 @@ detect_outliers <- function(data,
   # Merge features into matrix
   feat.internal <- cbind(org.series, gradient, abs.gradient, rel.gradient, seas_feat_mat)
   colnames(feat.internal)[5:ncol(feat.internal)] <- names
-  
+
   # TODO Allow users to choose features or to introduce new features
 
   # scale features
@@ -294,6 +304,7 @@ detect_outliers <- function(data,
   time.feature.inf.list <- list()
   feature.list <- list()
   for(j in 1:repetitions){
+
     # 5.) Subsetting of data
     zz_ss_ind <- sample(seq_len(nrow(data.original)),
                         ceiling(share * nrow(data.original)), replace = FALSE)
@@ -343,7 +354,7 @@ detect_outliers <- function(data,
       # Set deterministic features equal to zero
 
       exp.gri_base[,seq(5,ncol(feature.mat),2)[1:length(S)]] <- 0
-      # Use only those feautre combinations where mean shift 
+      # Use only those feautre combinations where mean shift
 
       # is reasonable...
       exp.gri <- unique(exp.gri_base)
@@ -455,10 +466,10 @@ detect_outliers <- function(data,
   probs.outlier.av <- apply(probs.mat,1,mean)
   outlier.pos.av <- which(probs.outlier.av > proba) # Final outliers based on averaging and above proba threshold
 
-  #outlier.pos augmented 
+  #outlier.pos augmented
   outlier.pos.aug <- as.vector(unique(sapply(outlier.pos.av, function(x) pmax(x-ext.val,1):pmin(x+ext.val, nrow(data.original)))))
   if(length(outlier.pos.aug)==0) outlier.pos.aug <- NULL
-  
+
   # feature grid
   if( length(outlier.vec.rep ) != 0 & feat.inf == T){
     exp.grid <- apply(exp.gri,1,function(x) names(which(x == 1)))
